/*****************************************************************************/
/*
                               ODSstruct.h


VERSION HISTORY
---------------
23-DEC-1999  MGD  initial; provide support for ODS-2 and ODS-5
*/
/*****************************************************************************/

#ifndef ODSSTRUCT_H_LOADED
#define ODSSTRUCT_H_LOADED 1

/* VMS related header files */
#include <descrip.h>
#include <rms.h>
#include <rmsdef.h>
#include <atrdef.h>
#include <fibdef.h>

/* application header files */
#include "enamel.h"

#ifdef __ALPHA
#  ifndef NO_ODS_EXTENDED
#     define ODS_EXTENDED 1
      /* this is smaller than the technical maximum, but still quite large! */
#     define ODS_MAX_FILE_NAME_LENGTH 511
#     define ODS_MAX_FILESYS_NAME_LENGTH 264
#  endif
#endif
#define ODS2_MAX_FILE_NAME_LENGTH 255
#ifndef ODS_MAX_FILE_NAME_LENGTH
#  define ODS_MAX_FILE_NAME_LENGTH ODS2_MAX_FILE_NAME_LENGTH
#endif
#if ODS_MAX_FILE_NAME_LENGTH < ODS2_MAX_FILE_NAME_LENGTH
#  define ODS_MAX_FILE_NAME_LENGTH ODS2_MAX_FILE_NAME_LENGTH
#endif

#ifdef __ALPHA
#   pragma __member_alignment __save
#   pragma nomember_alignment
#endif

typedef struct FileRecordAttributes FILE_RECORD_ATTR;

struct FileRecordAttributes
{
   unsigned char  RecordType;
   unsigned char  OfNoInterest1;
   unsigned short  RecordSize;
   unsigned short  AllocatedVbnHi;
   unsigned short  AllocatedVbnLo;
   unsigned short  EndOfFileVbnHi;
   unsigned short  EndOfFileVbnLo;
   unsigned short  FirstFreeByte;
   unsigned char   OfNoInterest3 [18];
};

typedef struct FileAcpStruct FILE_ACP_STRUCT;

struct FileAcpStruct
{
   unsigned short  Channel,
                   AtrFpro;
   unsigned long  AtrUic;
   unsigned long  CdtBinTime [2],
                  RdtBinTime [2];
   struct FileRecordAttributes  RecAttr;
   struct atrdef  FileAtr [6];
   struct dsc$descriptor  FileFibAcpDsc,
                          FileNameAcpDsc;
   struct AnIOsb  AcpIOsb;

#ifdef ODS_EXTENDED
   /* only use the local ENAMEL.H FIB definition when necessary */
#  ifdef ENAMEL_FIBDEF
      struct enamel_fibdef  FileFib;
#  else
      struct fibdef  FileFib;
#  endif
#else /* ODS_EXTENDED */
   struct fibdef  FileFib;
#endif /* ODS_EXTENDED */
};

typedef struct OdsStruct ODS_STRUCT;

struct OdsStruct
{
   BOOL  DeleteOnClose,
         NamlInUse,
         ParseInUse;

   int  DataLength,
        DataLineLength,
        DataLineNumber,
        DataLineSize,
        ExpFileNameLength,
        NamDeviceLength,
        NamDirectoryLength,
        NamFileSysNameLength,
        NamNameLength,
        NamNodeLength,
        NamTypeLength,
        NamVersionLength;

   unsigned long   Nam_fnb;

   char  *DataPtr,
         *DataLinePtr,
         *DataParsePtr,
         *NamDevicePtr,
         *NamDirectoryPtr,
         *NamFileSysNamePtr,
         *NamNamePtr,
         *NamNodePtr,
         *NamTypePtr,
         *NamVersionPtr;

   char  ExpFileName [ODS_MAX_FILE_NAME_LENGTH+1];

#ifdef ODS_EXTENDED
   char  SysFileName [ODS_MAX_FILESYS_NAME_LENGTH+1];
#endif

   /*
      This union allows the CACHE.C module to more efficiently use this
      storage.  It places it's copy of the cache *path* in the resultant
      file name field which would otherwise be completely wasted.
   */

   variant_union {
      int  CachedPathLength;
      int  ResFileNameLength;
   } Overlay1;

   variant_union {
      char  CachedPath [ODS_MAX_FILE_NAME_LENGTH+1];
      char  ResFileName [ODS_MAX_FILE_NAME_LENGTH+1];
   } Overlay2;

   GENERAL_AST AstFunction;
   unsigned long AstParam;

   struct FAB  Fab;
#  ifdef ODS_EXTENDED
      variant_union {
         struct NAM  Nam;
         struct NAML Naml;
      } Overlay3;
#  else /* ODS_EXTENDED */
      struct NAM  Nam;
#  endif /* ODS_EXTENDED */
   struct RAB  Rab;
   struct XABDAT  XabDat;
   struct XABFHC  XabFhc;
   struct XABPRO  XabPro;

   struct FileAcpStruct  AcpInfo;
};

#ifdef __ALPHA
#   pragma __member_alignment __restore
#endif

#endif /* ODSSTRUCT_H_LOADED */

/****************************************************************************/

